@@ -14,13 +14,14 @@ pub struct Action {
 
 #[derive(Debug,Clone)]
 pub struct Email {
+    pub receivers:Vec<String>,
     pub tracking_id:String,
     pub unique_id:String,
     pub private_key:String,
     pub server_name:String,
     pub name:String,
     pub from:String,
-    pub to:String,
+    pub to:Vec<String>,
     pub cc:String,
     pub bcc:String,
     pub subject:String,
@@ -37,13 +38,14 @@ impl Email{
     #[allow(dead_code)]
     pub fn new() -> Email{
         Email{
+            receivers:Vec::new(),
             tracking_id:String::new(),
             unique_id:String::new(),
             private_key:String::new(),
             server_name:String::new(),
             name:String::new(),
             from:String::new(),
-            to:String::new(),
+            to:Vec::new(),
             cc:String::new(),
             bcc:String::new(),
             subject:String::new(),
@@ -65,7 +67,9 @@ impl Email{
     #[allow(dead_code)]
     pub fn name(&mut self,v:String){self.name = v;}
     #[allow(dead_code)]
-    pub fn to(&mut self,v:String){self.to = v;}
+    pub fn receiver(&mut self,v:String){self.receivers.push(v);}
+    #[allow(dead_code)]
+    pub fn to(&mut self,v:String){self.to.push(v);}
     #[allow(dead_code)]
     pub fn from(&mut self,v:String){self.from = v;}
     #[allow(dead_code)]
