@@ -3,11 +3,13 @@ use rand::{thread_rng, Rng};
 use rand::distributions::Alphanumeric;
 use crate::client::io::read_file;
 // use crate::client::io::write_file;
-use crate::client::Connection;
+use crate::client::{Connection};
 use openssl::sha::Sha256;
 use openssl::sign::Signer;
 use openssl::pkey::{PKey,Private};
 use openssl::hash::MessageDigest;
+use quoted_printable::encode_to_str as QPEncoder;
+// use base64::encode as Base64Encoder;
 
 pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64),&'static str>{
 
@@ -110,8 +112,14 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
         body.push_str(&format!("{}\r\n",clean_text(&email.body)));
         body.push_str(&format!("--000000000000dbc95d05d2847225\r\n"));
         body.push_str(&format!(r#"Content-Type: text/html; charset="UTF-8""#));
+        // body.push_str(&format!(r#"Content-Type: text/html; charset="US-ASCII""#));
+        body.push_str(&format!("\r\nContent-Transfer-Encoding: quoted-printable"));
+        // body.push_str(&format!("\r\nContent-Transfer-Encoding: base64"));
+        // println!("where");
+        // body.push_str(&"\r\n");
+        // body.push_str(&format!("\r\nContent-Transfer-Encoding: 8BIT"));
         body.push_str(&"\r\n");body.push_str(&"\r\n");
-        body.push_str(&format!("{}\r\n",clean_text(&email.html)));
+        body.push_str(&format!("{}\r\n",clean_html(&email.html)));
         if with_files{
             body.push_str(&format!("--000000000000dbc95d05d2847225--\r\n"));
         }
@@ -125,7 +133,7 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
         if email.html.len() > 0{
             body.push_str(&format!(r#"Content-Type: text/plain; charset="UTF-8""#));
             body.push_str(&"\r\n");body.push_str(&"\r\n");
-            body.push_str(&format!("{}\r\n",clean_text(&email.html)));
+            body.push_str(&format!("{}\r\n",clean_html(&email.html)));
         }
     }
     if with_files{
@@ -154,10 +162,12 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
         body.push_str(&format!("{}\r\n",clean_text(&email.body)));
     }
     if !with_files && !alternatives && only_html{
-        body.push_str(&format!("{}\r\n",clean_text(&email.html)));
+        body.push_str(&format!("{}\r\n",clean_html(&email.html)));
     }
 
-    // println!("{}",body);
+    // println!("\n-----------\n{:?}\n-----------\n",body);
+
+    body = clean_bytes_from_string(body);
 
     dkim.push_str(&"v=1;");
     dkim.push_str(&" a=rsa-sha256;");
@@ -172,11 +182,12 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
     let dkim_signature_string = dkim_headers + "dkim-signature:" + &dkim;
 
     // println!("\n-------\n\n{}\n\n-------\n",headers);
-    // println!("\n-------\n\n{}\n\n-------\n",body);
-    // println!("\n-------\n\n{:?}\n\n-------\n",dkim_signature_string);
+    // println!("\n-------\n\n{:?}\n\n-------\n",body);
+    // println!("\n-------\n\n dkim_signature_string : {:?}\n\n-------\n",dkim_signature_string);
 
     match sign_here(&conn.private_key,dkim_signature_string.into_bytes()){
         Ok(dkim_signature)=>{
+            // println!("\n-------\n\n dkim_signature : {:?}\n\n-------\n",dkim_signature);
             dkim = dkim.replace("b=",&format!("b={}",dkim_signature));
         },
         Err(_)=>{
@@ -187,9 +198,11 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
     headers = headers.replace("===dkim===",&dkim);
     commands.push(format!("{}\r\n{}\r\n.\r\n",headers,body));
 
+    // println!("{:?}",commands);
+
     // match crate::io::write_file(
-    //     "D:\\workstation\\expo\\rust\\letterman\\letterman_email_body_parser\\ge_html_ww.txt".to_string(),
-    //     commands[3].clone().as_bytes().to_vec()
+    //     "D:\\workstation\\expo\\rust\\letterman\\letterman_email_body_parser\\ge_html_wwe.txt".to_string(),
+    //     commands[4].clone().as_bytes().to_vec()
     // ).await{
     //     Ok(_)=>{
     //         println!(">>>>>>>>>>>>>>>>> write complete");
@@ -210,33 +223,64 @@ pub async fn init(email:Email,conn:&Connection)->Result<(Vec<String>,String,u64)
     size += commands[2].len() as u64;
     size += commands[3].len() as u64;
 
-    // println!("{:?}",commands);
+    // println!("{}",commands[3]);
 
     return Ok((commands,email.tracking_id,size));
 
 }
 
+fn clean_bytes_from_string(body:String)->String{
+
+    if true{
+        return body;
+    }
+
+    let as_bytes = body.as_bytes().to_vec();
+    let chars:Vec<char> = body.chars().collect();
+
+    println!("len1 : {}",chars.len());
+    println!("len2 : {}",as_bytes.len());
+
+    if false{
+        let mut index = 0;
+        for c in chars{
+            if as_bytes[index] < 32 || as_bytes[index] > 126{
+                print!("{} {:?} {} ,\n",index,c,as_bytes[index]);
+            }
+            index += 1;
+        }
+    }
+
+    match String::from_utf8(as_bytes){
+        Ok(v)=>{
+            println!("{}",v);
+            return v;
+        },
+        Err(_)=>{
+            return String::new();
+        }
+    }
+
+    // return body;
+
+}
+
 fn clean_text(text:&String)->String{
     let base = text.to_string();
     let clean_1 = base.trim_start().to_string();
     let mut build = clean_1.trim_end().to_string();
-    loop{
-        if !build.contains("\r\n"){
-            break;
-        }
-        build = build.replace("\r\n","+999++9+999+9+9+999+9");
-    }
-    loop{
-        if !build.contains("\n"){
-            break;
-        }
-        build = build.replace("\n","+999++9+999+9+9+999+9");
-    }
+    build = replace_text(build, "\r\n", "+999++9+999+9+9+999+9");
+    build = replace_text(build, "\n", "+999++9+999+9+9+999+9");
     let pool:Vec<&str> = build.split("+999++9+999+9+9+999+9").collect();
     let mut rebuild = String::new();
     for mut i in pool{
         i = i.trim_start();
+        i = i.trim_end();
         if i.len() > 0{
+            // let h = ;
+            let k = &build_line(i.to_string());
+            i = k;
+            // println!("{:?}",i);
             if rebuild.len() > 0{
                 rebuild.push_str(&format!("\r\n{}",i));
             } else {
@@ -244,9 +288,43 @@ fn clean_text(text:&String)->String{
             }
         }
     }
+    rebuild = replace_text(rebuild, "\r\n.\r\n", ".\r\n");
     return rebuild;
 }
 
+fn clean_html(text:&String)->String{
+    let mut base = clean_text(text);
+    base = replace_text(base, "\r\n", "");
+    base = build_line(base);
+    base = QPEncoder(base);
+    return base;
+}
+
+fn replace_text(mut text:String,find:&'static str,replace:&'static str)->String{
+    loop{
+        if !text.contains(find){
+            break;
+        }
+        text = text.replace(find,replace);
+    }
+    return text;
+}
+
+fn build_line(mut m:String)->String{
+    let mut build = String::new();
+    loop{
+        if m.len() > 1000{
+            let (first, last) = m.split_at(990);
+            build.push_str(&format!("{}\r\n",first));
+            m = last.to_string();
+        } else {
+            build.push_str(&format!("{}",m));
+            break;
+        }
+    }
+    return build;
+}
+
 async fn parse_files(files:Vec<String>)->Result<String,&'static str>{
     let mut build = String::new();
     for path in files.iter(){
@@ -258,7 +336,7 @@ async fn parse_files(files:Vec<String>)->Result<String,&'static str>{
                 build.push_str(&format!(r#"Content-Disposition: attachment; filename="{}""#,&file.0));
                 build.push_str(&"\r\n");
                 build.push_str(&format!("Content-Transfer-Encoding: base64\r\n\r\n"));
-                build.push_str(&format!("{}\r\n",file.2));
+                build.push_str(&format!("{}\r\n",build_line(file.2)));
             },
             Err(_)=>{
                 return Err("failed-read_file");
@@ -277,7 +355,7 @@ fn parse_base64_attachments(attachments:Vec<(String,String,String)>)->String{
         build.push_str(&format!(r#"Content-Disposition: attachment; filename="{}""#,&attachment.0));
         build.push_str(&"\r\n");
         build.push_str(&format!("Content-Transfer-Encoding: base64\r\n\r\n"));
-        build.push_str(&format!("{}\r\n",attachment.1));
+        build.push_str(&format!("{}\r\n",build_line(attachment.1.clone())));
     }
     return build;
 }
@@ -293,6 +371,7 @@ fn unique_id(size:usize)->String{
 
 fn hash_sha256(v:String) -> String{
     let mut hasher = Sha256::new();
+    // println!("{}",v);
     hasher.update(&v.into_bytes());
     let hash = hasher.finish();
     let encode = base64::encode(&hash);
