@@ -12,12 +12,17 @@ use letterman_email_body_parser::init as EmailBodyParser;
 use crate::server::config::{ServerInfo};
 use crate::server::io;
 use crate::server::config::{QueMessage,CheckMail,Email,QueAddMessage,Signal};
-use crate::server::{BAD_AUTH,DOWN,OK_REPLY,BAD_SEQUENCE,USER_NOT_FOUND,DATA_START,QUIT_REPLY};
+use crate::server::{
+    BAD_AUTH,DOWN,OK_REPLY,BAD_SEQUENCE,USER_NOT_FOUND,
+    DATA_START,QUIT_REPLY,NO_RECEIVER_FOUND,UNHANDLED,BAD_PARAMS
+};
 use crate::server::jsonify::init as jasonify_init;
 use letterman_email_body_parser::EmailBody;
 use flume::Sender as FlumeSender;
 use json::JsonValue;
 
+const DATA_END:&'static str = "\r\n.\r\n";
+
 pub async fn error(stream:TokioRustLsServerTlsStream<TcpStream>,e:&'static str){
     let mut stream = stream;
     match io::send_server(&mut stream,&format!("{}",e)).await{
@@ -56,6 +61,8 @@ where
     let mut que_sender = que_sender;
     let info_lock = info.read().await;
 
+    // println!("### connection started");
+
     //------------------------------
     //say hello
     //------------------------------
@@ -68,6 +75,8 @@ where
         }
     }
 
+    // println!("### helo sent");
+
     //get ehlo response
     let server_name:String;
     match io::read_server(&mut stream,&info_lock.regex.ehlo).await{
@@ -111,6 +120,7 @@ where
     // features.push_str(&format!("250-{} Hello {}\r\n",&info_lock.name,&hostname));
     features.push_str(&format!("250-SIZE {}\r\n",&info_lock.size));
     features.push_str(&format!("250-PIPELINING\r\n"));
+    features.push_str(&format!("250-8BITMIME\r\n"));
     match io::send_server(&mut stream,&features).await{
         Ok(_)=>{},
         Err(_)=>{
@@ -129,89 +139,19 @@ where
 
     loop{
 
+        if email.status == 3{email.status = 4;}
         let hold_check_mail_sender = check_mail_sender.clone();
 
-        //from message
-        if email.status == 0{
-            match io::async_server_read_till(
+        //read message
+        let message:String;
+        if email.status == 4{
+            match io::read_server_till_end(
                 &mut stream,
-                &info_lock.message_end,
-                &mut buffer,max_size_local
+                &mut buffer,
+                &DATA_END
             ).await{
                 Ok(request_string)=>{
-                    if request_string.contains("MAIL FROM"){
-                        // println!("from : {:?}",request_string);
-                        match info_lock.regex.email.captures(&request_string){
-                            Some(captures)=>{
-                                if info_lock.validate_spf{
-                                    match captures.get(2){
-                                        Some(v)=>{
-                                            //check if host can even send this email
-                                            match TokioSpfCheck(
-                                                &info_lock.spf_config,
-                                                ip.clone(),
-                                                &server_name,
-                                                &v.as_str().to_string()
-                                            ).await{
-                                                Ok(query_result)=>{
-                                                    email.spf = query_result;
-                                                },
-                                                Err(_)=>{
-                                                    email.reset();
-                                                    match send_message(&mut stream, DOWN).await{
-                                                        Ok(_)=>{},Err(_)=>{break;}
-                                                    }
-                                                }
-                                            }
-                                        },
-                                        None=>{
-                                            email.reset();
-                                            match send_message(&mut stream, BAD_SEQUENCE).await{
-                                                Ok(_)=>{},Err(_)=>{break;}
-                                            }
-                                        }
-                                    }//get email domain match from regex
-                                }
-                                match captures.get(0){
-                                    Some(v)=>{
-                                        email.reset();
-                                        email.from = v.as_str().to_string();
-                                        email.status = 1;
-                                        match send_message(&mut stream, OK_REPLY).await{
-                                            Ok(_)=>{},Err(_)=>{break;}
-                                        }
-                                    },
-                                    None=>{
-                                        email.reset();
-                                        match send_message(&mut stream, BAD_SEQUENCE).await{
-                                            Ok(_)=>{},Err(_)=>{break;}
-                                        }
-                                    }
-                                }//get full email match from regex
-                            },
-                            None=>{
-                                email.reset();
-                                match send_message(&mut stream, BAD_SEQUENCE).await{
-                                    Ok(_)=>{},Err(_)=>{break;}
-                                }
-                            }//no email regex found in requets stirng
-                        }
-                    } else if request_string.contains("RSET"){
-                        email.reset();
-                        match send_message(&mut stream, OK_REPLY).await{
-                            Ok(_)=>{},Err(_)=>{break;}
-                        }
-                    } else if request_string.contains("QUIT"){
-                        match send_message(&mut stream, QUIT_REPLY).await{
-                            Ok(_)=>{},Err(_)=>{}
-                        }
-                        break;
-                    } else {
-                        email.reset();
-                        match send_message(&mut stream, BAD_SEQUENCE).await{
-                            Ok(_)=>{},Err(_)=>{break;}
-                        }
-                    }//no request string match found
+                    message = request_string;
                 },
                 Err(_)=>{
                     println!("failed async_server_read_till");
@@ -219,144 +159,195 @@ where
                     break;
                 }//read failed on connection
             }
-        }//rcpt to request
-
-        //to message
-        if email.status == 1{
+        } else {
             match io::async_server_read_till(
                 &mut stream,
                 &info_lock.message_end,
                 &mut buffer,max_size_local
             ).await{
                 Ok(request_string)=>{
-                    if request_string.contains("RCPT TO"){
-                        // println!("to : {:?}",request_string);
-                        match info_lock.regex.email.captures(&request_string){
-                            Some(captures)=>{
-                                match captures.get(0){
-                                    Some(v)=>{
-                                        match check_email(CheckMail{
-                                            from:email.from.clone(),
-                                            to:email.to.clone()
-                                        },hold_check_mail_sender).await{
-                                            Ok(result)=>{
-                                                if result == true{
-                                                    match send_message(&mut stream, OK_REPLY).await{
-                                                        Ok(_)=>{},Err(_)=>{break;}
-                                                    }
-                                                    email.to = v.as_str().to_string();
-                                                    email.status = 2;
-                                                } else {
-                                                    email.reset();
-                                                    match send_message(&mut stream, USER_NOT_FOUND).await{
-                                                        Ok(_)=>{},Err(_)=>{break;}
-                                                    }
-                                                }
-                                            },
-                                            Err(_)=>{
-                                                email.reset();
-                                                match send_message(&mut stream, BAD_SEQUENCE).await{
-                                                    Ok(_)=>{},Err(_)=>{break;}
-                                                }
+                    message = request_string;
+                },
+                Err(_)=>{
+                    println!("failed async_server_read_till");
+                    error(stream,DOWN).await;
+                    break;
+                }//read failed on connection
+            }
+        }
+
+        if email.status < 3{
+            if message.contains("MAIL FROM:") || message.contains("MAIL FROM :"){
+                // println!("MAIL FROM");
+                email.reset();
+                let split_message:Vec<&str> = message.split(":").collect();
+                match info_lock.regex.email.captures(&split_message[1]){
+                    Some(captures)=>{
+                        if info_lock.validate_spf{
+                            match captures.get(3){
+                                Some(v)=>{
+                                    //check if host can even send this email
+                                    let domain = v.as_str().to_string();
+                                    match TokioSpfCheck(
+                                        &info_lock.spf_config,
+                                        ip.clone(),
+                                        &server_name,
+                                        &domain
+                                    ).await{
+                                        Ok(query_result)=>{
+                                            email.spf = query_result;
+                                        },
+                                        Err(_)=>{
+                                            email.reset();
+                                            match send_message(&mut stream, DOWN).await{
+                                                Ok(_)=>{},Err(_)=>{break;}
                                             }
                                         }
-                                    },
-                                    None=>{
-                                        email.reset();
-                                        match send_message(&mut stream, BAD_SEQUENCE).await{
-                                            Ok(_)=>{},Err(_)=>{break;}
-                                        }
-                                    }//get matching email from captures
+                                    }
+                                },
+                                None=>{
+                                    email.reset();
+                                    match send_message(&mut stream, BAD_PARAMS).await{
+                                        Ok(_)=>{},Err(_)=>{break;}
+                                    }
+                                }
+                            }//get email domain match from regex
+                        }
+                        match captures.get(1){
+                            Some(v)=>{
+                                email.reset();
+                                email.sender = v.as_str().to_string();
+                                email.status = 1;
+                                match send_message(&mut stream, OK_REPLY).await{
+                                    Ok(_)=>{},Err(_)=>{break;}
                                 }
                             },
                             None=>{
                                 email.reset();
-                                match send_message(&mut stream, BAD_SEQUENCE).await{
+                                match send_message(&mut stream, BAD_PARAMS).await{
                                     Ok(_)=>{},Err(_)=>{break;}
                                 }
-                            }//no email captured in regex
-                        }
-                    } else {
-                        match send_message(&mut stream, BAD_SEQUENCE).await{
+                            }
+                        }//get full email match from regex
+                    },
+                    None=>{
+                        email.reset();
+                        match send_message(&mut stream, BAD_PARAMS).await{
                             Ok(_)=>{},Err(_)=>{break;}
                         }
-                    }//no requets string match found
-                },
-                Err(_)=>{
-                    println!("failed async_server_read_till");
-                    error(stream,DOWN).await;
-                    break;
+                    }//no email regex found in requets stirng
                 }
-            }//reader
-        }//email to request
-
-        //data message
-        if email.status == 2{
-            match io::async_server_read_till(
-                &mut stream,
-                &info_lock.message_end,
-                &mut buffer,max_size_local
-            ).await{
-                Ok(request_string)=>{
-                    if request_string.contains("DATA"){
-                        // println!("data : {:?}",request_string);
-                        match send_message(&mut stream, DATA_START).await{
-                            Ok(_)=>{
-                                email.status = 3;
-                            },Err(_)=>{break;}
-                        }
-                    } else {
-                        match send_message(&mut stream, BAD_SEQUENCE).await{
-                            Ok(_)=>{},Err(_)=>{break;}
-                        }
-                    }//no requets string match found
-                },
-                Err(_)=>{
-                    println!("failed async_server_read_till");
-                    error(stream,DOWN).await;
-                    break;
-                }
-            }//reader
-        }//email to request
-
-        //data message
-        //from message
-        if email.status == 3{
-            // println!("where");
-            match io::async_server_read_till(
-                &mut stream,
-                &info_lock.data_end,
-                &mut buffer,max_size_local
-            ).await{
-                Ok(request_string)=>{
-                    // println!(">>> parsing email");
-                    let hold:Vec<&str> = request_string.split("\r\n").collect();
-                    match EmailBodyParser(hold,&info_lock.email_body_parser_config){
-                        Ok(mut email_body)=>{
-                            if info_lock.validate_dkim{//if dkim validation is turned on
-                                match email_body.validate(&info_lock.email_body_parser_config).await{
-                                    Ok(_)=>{
-                                        match process_mail(
-                                            &mut stream,
-                                            email.flush(),
-                                            email_body,
-                                            &mut que_sender,
-                                            &attachment_storage_dir
-                                        ).await{
-                                            Ok(_)=>{
-                                                email.reset();
-                                            },
-                                            Err(_)=>{break;}
-                                        }
-                                    },
-                                    Err(_)=>{//email dkim validation failed
-                                        match send_message(&mut stream, BAD_AUTH).await{
-                                            Ok(_)=>{},Err(_)=>{break;}
+            } else if message.contains("RCPT TO:") || message.contains("RCPT TO :"){
+                // println!("RCPT TO");
+                if email.sender.len() == 0{
+                    email.reset();
+                    match send_message(&mut stream, BAD_SEQUENCE).await{
+                        Ok(_)=>{email.status = 0;},Err(_)=>{break;}
+                    }
+                } else {
+                    let split_message:Vec<&str> = message.split(":").collect();
+                    match info_lock.regex.email.captures(&split_message[1]){
+                        Some(captures)=>{
+                            match captures.get(1){
+                                Some(v)=>{
+                                    let to = v.as_str().to_string();
+                                    match check_email(CheckMail{
+                                        from:email.sender.clone(),
+                                        to:to.clone()
+                                    },hold_check_mail_sender).await{
+                                        Ok(result)=>{
+                                            if result == true{
+                                                match send_message(&mut stream, OK_REPLY).await{
+                                                    Ok(_)=>{},Err(_)=>{break;}
+                                                }
+                                                email.receivers.push(to); 
+                                                email.status = 2;
+                                            } else {
+                                                // email.reset();
+                                                match send_message(&mut stream, USER_NOT_FOUND).await{
+                                                    Ok(_)=>{},Err(_)=>{break;}
+                                                }
+                                            }
+                                        },
+                                        Err(_)=>{
+                                            email.reset();
+                                            match send_message(&mut stream, DOWN).await{
+                                                Ok(_)=>{},Err(_)=>{break;}
+                                            }
                                         }
                                     }
-                                }//validate dkim 
-                            } else {//if dkim validation is turned off
+                                },
+                                None=>{
+                                    email.reset();
+                                    match send_message(&mut stream, BAD_PARAMS).await{
+                                        Ok(_)=>{},Err(_)=>{break;}
+                                    }
+                                }//get matching email from captures
+                            }
+                        },
+                        None=>{
+                            email.reset();
+                            match send_message(&mut stream, BAD_PARAMS).await{
+                                Ok(_)=>{},Err(_)=>{break;}
+                            }
+                        }//no email captured in regex
+                    }
+                }
+            } else if message.contains("DATA"){
+                // println!("DATA");
+                if !email.check(){
+                    email.reset();
+                    match send_message(&mut stream, BAD_SEQUENCE).await{
+                        Ok(_)=>{email.status = 0;},Err(_)=>{break;}
+                    }
+                } else {
+                    if email.receivers.len() == 0{
+                        email.reset();
+                        match send_message(&mut stream, NO_RECEIVER_FOUND).await{
+                            Ok(_)=>{email.status = 0;},Err(_)=>{break;}
+                        }
+                    } else {
+                        match send_message(&mut stream, DATA_START).await{
+                            Ok(_)=>{email.status = 3;},Err(_)=>{break;}
+                        }
+                    }
+                }
+            } else if message.contains("RSET"){
+                // println!("RSET");
+                email.reset();
+                match send_message(&mut stream, OK_REPLY).await{
+                    Ok(_)=>{},Err(_)=>{break;}
+                }
+            } else if message.contains("QUIT"){
+                // println!("QUIT");
+                match send_message(&mut stream, QUIT_REPLY).await{
+                    Ok(_)=>{},Err(_)=>{}
+                }
+                break;
+            } else {
+                // println!("UNHANDLED : {:?}",message);
+                email.reset();
+                match send_message(&mut stream, UNHANDLED).await{
+                    Ok(_)=>{},Err(_)=>{break;}
+                }
+            }
+        }//check messages till data
+
+        if email.status == 4{
+            // println!("BODY");
+            // println!("{:?}",message);
+            let hold:Vec<&str> = message.split("\r\n").collect();
+            // if hold[hold.len()-1].len() == 0{
+            //     hold.remove(hold.len()-1);
+            // }
+            // println!("\n{:?}\n",hold);
+            match EmailBodyParser(hold,&info_lock.email_body_parser_config){
+                Ok(mut email_body)=>{
+                    if info_lock.validate_dkim{//if dkim validation is turned on
+                        match email_body.validate(&info_lock.email_body_parser_config).await{
+                            Ok(_)=>{
                                 match process_mail(
+                                    // &info,
                                     &mut stream,
                                     email.flush(),
                                     email_body,
@@ -368,36 +359,56 @@ where
                                     },
                                     Err(_)=>{break;}
                                 }
-                            }//if dkim validation is turned off
-                        },//email body parsed
-                        Err(_)=>{
-                            match send_message(&mut stream, BAD_SEQUENCE).await{
-                                Ok(_)=>{},Err(_)=>{break;}
+                            },
+                            Err(_)=>{//email dkim validation failed
+                                match send_message(&mut stream, BAD_AUTH).await{
+                                    Ok(_)=>{},Err(_)=>{break;}
+                                }
                             }
+                        }//validate dkim 
+                    } else {//if dkim validation is turned off
+                        match process_mail(
+                            // &info,
+                            &mut stream,
+                            email.flush(),
+                            email_body,
+                            &mut que_sender,
+                            &attachment_storage_dir
+                        ).await{
+                            Ok(_)=>{
+                                email.reset();
+                            },
+                            Err(_)=>{break;}
                         }
-                    }//parse email body 
-
-                },//found \r\n.\r\n
+                    }//if dkim validation is turned off
+                },//email body parsed
                 Err(_)=>{
-                    println!("failed async_server_read_till");
-                    error(stream,DOWN).await;
-                    break;
+                    match send_message(&mut stream, BAD_PARAMS).await{
+                        Ok(_)=>{},Err(_)=>{break;}
+                    }
                 }
-            }//read email body
-        }//data input message
+            }//parse email body 
+            email.status = 0;
+        }
 
-    }
+    }//loop message
 
 }
 
 async fn process_mail(
+    // info:&Arc<RwLock<ServerInfo>>,
     stream:&mut TokioRustLsServerTlsStream<TcpStream>,
     email:Email,
     body:EmailBody,
     que_sender:&mut Sender<QueMessage>,
     attachment_storage_dir:&String
 )->Result<(),()>{
-    match jasonify_init(email,body,attachment_storage_dir).await{
+    match jasonify_init(
+        // info,
+        email,
+        body,
+        attachment_storage_dir
+    ).await{
         Ok(body)=>{
 
             let (signal,sleeper) = Signal::new();
